package net.knarcraft.minecraftserverlauncher.userinterface;

import javax.swing.*;
import java.awt.*;

/**
 * This class keeps track of all consoles
 *
 * <p>A parent window for server consoles. Should be toggled with the "View server consoles" button.
 * Keeps track of all consoles.</p>
 *
 * @author Kristian Knarvik <kristian.knarvik@knett.no>
 * @version 1.0.0
 * @since 1.0.0
 */
public class ServerConsoles {
    private static JFrame frame;
    private static JTabbedPane consolesTabbedPane;

    /**
     * Initializes the server consoles frame
     */
    public ServerConsoles() {
        frame = new JFrame();
        frame.setBounds(100, 100, 450, 300);
        frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
        consolesTabbedPane = new JTabbedPane(JTabbedPane.TOP);
        frame.getContentPane().add(consolesTabbedPane, BorderLayout.CENTER);
    }

    /**
     * Adds a new console tab
     *
     * @param name <p>The name of the consoles tab</p>
     * @return <p>A new console element with the new tabbed pane</p>
     */
    public static Console addConsoleTab(String name) {
        return new Console(consolesTabbedPane, name);
    }

    /**
     * Sets the server consoles frame as visible
     */
    public static void setAsVisible() {
        frame.setVisible(true);
    }

    /**
     * Returns the tabbed pane containing the server consoles
     *
     * @return <p>The tabbed pane containing the server consoles</p>
     */
    public static JTabbedPane getTabbedPane() {
        return consolesTabbedPane;
    }
}